import requests


class RequestFactory(object):
    jwt_token = None
    basic_auth = None

    def __init__(self, jwt_token=None, basic_auth=None):
        self.jwt_token = jwt_token
        self.basic_auth = basic_auth
        super().__init__()

    def get(self, *args, **kwargs):
        if self.jwt_token is not None:
            result = self._get_using_jwt(*args, **kwargs)
            if result is not None:
                return result

        if self.basic_auth is not None:
            result = self._get_using_basic_auth(*args, **kwargs)
            if result is not None:
                return result

        result = self._get_no_auth(*args, **kwargs)
        if result is not None:
            return result

        # I'm not even sure it's possible to ever get here, but it's good to cover all bases.
        raise RuntimeError('Unable to obtain a result. No gets were got.')

    def _get_using_jwt(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'JWT ' + self.jwt_token
        kwargs['headers'] = headers
        return requests.get(*args, **kwargs)

    def _get_using_basic_auth(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'Basic ' + self.basic_auth
        kwargs['headers'] = headers
        return requests.get(*args, **kwargs)

    def _get_no_auth(self, *args, **kwargs):
        return requests.get(*args, **kwargs)

    def post(self, *args, **kwargs):
        if self.jwt_token is not None:
            result = self._post_using_jwt(*args, **kwargs)
            if result is not None:
                return result

        if self.basic_auth is not None:
            result = self._post_using_basic_auth(*args, **kwargs)
            if result is not None:
                return result

        result = self._post_no_auth(*args, **kwargs)
        if result is not None:
            return result

        # I'm not even sure it's possible to ever get here, but it's good to cover all bases.
        raise RuntimeError('Unable to obtain a result. No posts were posted.')

    def _post_using_jwt(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'JWT ' + self.jwt_token
        kwargs['headers'] = headers
        return requests.post(*args, **kwargs)

    def _post_using_basic_auth(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'Basic ' + self.basic_auth
        kwargs['headers'] = headers
        return requests.post(*args, **kwargs)

    def _post_no_auth(self, *args, **kwargs):
        return requests.post(*args, **kwargs)

    def delete(self, *args, **kwargs):
        if self.jwt_token is not None:
            result = self._delete_using_jwt(*args, **kwargs)
            if result is not None:
                return result

        if self.basic_auth is not None:
            result = self._delete_using_basic_auth(*args, **kwargs)
            if result is not None:
                return result

        result = self._delete_no_auth(*args, **kwargs)
        if result is not None:
            return result

        # I'm not even sure it's possible to ever get here, but it's good to cover all bases.
        raise RuntimeError('Unable to obtain a result. No deletes were deleted.')

    def _delete_using_jwt(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'JWT ' + self.jwt_token
        kwargs['headers'] = headers
        return requests.delete(*args, **kwargs)

    def _delete_using_basic_auth(self, *args, **kwargs):
        headers = kwargs.get('headers', dict())
        headers['Authorization'] = 'Basic ' + self.basic_auth
        kwargs['headers'] = headers
        return requests.delete(*args, **kwargs)

    def _delete_no_auth(self, *args, **kwargs):
        return requests.delete(*args, **kwargs)



from rest_framework_jwt.authentication import JSONWebTokenAuthentication
from rest_framework import exceptions


class UserProxy(object):
    """This class emulates a Django user model,
    acting as a mapping layer between the token
    payload and user data.
    """

    # Django expects these fields to exist on a "user model"
    pk = None
    is_authenticated = True
    is_anonymous = False

    def __init__(self,
                 email,
                 uuid,
                 groups,
                 primary_group):
        super(UserProxy, self).__init__()

        if groups is not None and not hasattr(groups, '__iter__'):
            msg = 'Groups field must be an iterable or null'
            raise exceptions.AuthenticationFailed(msg)

        self.email = email
        self.uuid = uuid
        self.groups = groups
        self.primary_group = primary_group

    def is_authenticated(self):
        return self.is_authenticated

    def is_anonymous(self):
        return self.is_anonymous


class MiniJWTAuthentication(JSONWebTokenAuthentication):

    def authenticate_credentials(self, payload):
        """If we've got this far it means that the JWT token has already been authenticated.
        Here we set request.user to a normal Python object instead of an Django model object read
        from the database.
        """

        email = payload.get('email')
        uuid = payload.get('user_id')
        groups = payload.get('groups')
        primary_group = payload.get('primary_group')

        return UserProxy(email=email,
                                uuid=uuid,
                                groups=groups,
                                primary_group=primary_group)




import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='mini-jwt-backend',
    version='17.8.1',
    packages=find_packages(),
    install_requires=[
        'djangorestframework',
        'djangorestframework-jwt',
        'requests'
    ],
    include_package_data=True,
    license='MIT',
    description='A simple JWT Django Auth backend',
    long_description=README,
    url='https://gitlab.com/chaosengine256/mini-jwt-backend',
    author='Chaos Engine',
    author_email='chaosengine256@gmail.com',
)
